@@ -1,14 +1,15 @@
 package Spreadsheet::HTML;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.22';
+our $VERSION = '0.23';
 
 use Exporter 'import';
 our @EXPORT_OK = qw(
     generate portrait landscape
-    north east south west layout
-    chess checkers checkerboard
-    dk
+    north east south west
+    layout checkerboard
+    chess checkers conway
+    dk shroom
 );
 
 use Clone;
@@ -26,10 +27,12 @@ sub south   { generate( @_, theta => -180, tgroups => 0, pinhead => 1 ) }
 sub west    { generate( @_, theta => -270, tgroups => 0 ) }
 
 sub layout          { Spreadsheet::HTML::Presets::layout( @_ ) }
+sub conway          { Spreadsheet::HTML::Presets::conway( @_ ) }
 sub chess           { Spreadsheet::HTML::Presets::chess( @_ ) }
 sub checkers        { Spreadsheet::HTML::Presets::checkers( @_ ) }
 sub checkerboard    { Spreadsheet::HTML::Presets::checkerboard( @_ ) }
 sub dk              { Spreadsheet::HTML::Presets::dk( @_ ) }
+sub shroom          { Spreadsheet::HTML::Presets::shroom( @_ ) }
 
 sub generate {
     my %args = _process( @_ );
@@ -91,9 +94,6 @@ sub _process {
         return wantarray ? ( data => $self->{data}, %{ $args || {} } ) : $data;
     }
 
-    my $empty = exists $args->{empty} ? $args->{empty} : '&nbsp;';
-    my $max_cols = scalar @{ $data->[0] };
-
     # headings is an alias for -row0
     $args->{-row0} = delete $args->{headings} if exists $args->{headings};
 
@@ -106,11 +106,13 @@ sub _process {
         }
     }
 
-    for my $row (0 .. $#$data) {
+    my $empty = exists $args->{empty} ? $args->{empty} : '&nbsp;';
+
+    for my $row (0 .. $args->{_max_rows} - 1) {
 
         unless ($args->{_layout}) {
-            push @{ $data->[$row] }, undef for 1 .. $max_cols - $#{ $data->[$row] } + 1;  # pad
-            pop  @{ $data->[$row] } for $max_cols .. $#{ $data->[$row] };                 # truncate
+            push @{ $data->[$row] }, undef for 1 .. $args->{_max_cols} - $#{ $data->[$row] } + 1;  # pad
+            pop  @{ $data->[$row] } for $args->{_max_cols} .. $#{ $data->[$row] };                 # truncate
         }
 
         for my $col (0 .. $#{ $data->[$row] }) {
@@ -230,29 +232,27 @@ sub _colgroup {
 }
 
 sub _args {
-    my ($self,$data,$args);
+    my ($self,@data,$data,@args,$args);
     $self = shift if UNIVERSAL::isa( $_[0], __PACKAGE__ );
+    $data = shift if (@_ == 1);
 
-    if (@_ > 1 && defined($_[0]) && !ref($_[0]) ) {
-        $args = {@_};
-        $data = delete $args->{data} if exists $args->{data};
-    } elsif (@_ > 1 && ref($_[0]) eq 'ARRAY') {
-        if (ref($_[0]->[0]) eq 'ARRAY') {
-            $data = shift;
-        }
-        my @args;
-        for (@_) {
-            if (ref($_) eq 'ARRAY') {
-                push @$data, $_;
-            } else {
-                push @args, $_; 
+    while (@_) {
+        if (ref( $_[0] )) {
+            push @data, shift;
+            if (ref( $_[0] )) {
+                push @data, shift;
+            } elsif (defined $_[0]) {
+                push @args, shift, shift;
             }
+        } else {
+            push @args, shift, shift;
         }
-        $args = {@args};
-    } elsif (@_ == 1) {
-        $data = $_[0];
     }
 
+    $data ||= (@data == 1) ? $data[0] : (@data) ? [ @data ] : undef;
+    $args = scalar @args ? { @args } : {};
+    $data = delete $args->{data} if exists $args->{data};
+
     if ($self) {
         return ( $self, $self->{data}, $args ) if $self->{is_cached};
         $args = { %{ $self || {} }, %{ $args || {} } };
@@ -260,22 +260,22 @@ sub _args {
         $data = $self->{data} unless $data or $args->{file};
     }
 
-    $data = _fill_data( $args ) if $args->{fill};
     $data = Spreadsheet::HTML::File::Loader::parse( $args->{file} ) if $args->{file};
     $data = [ $data ] unless ref($data);
     $data = [ $data ] unless ref($data->[0]);
     $data = [ [undef] ] if !scalar @{ $data->[0] };
 
-    return ( $self, Clone::clone($data), $args );
-}
+    my %fill;
+    if ($args->{fill}) {
+        ($fill{row},$fill{col}) = $args->{fill} =~ /^(\d+)\D(\d+)$/;
+    }
 
-sub _fill_data {
-    my $args = shift;
-    my ($rows,$cols) = $args->{fill} =~ /(\d+)\D+(\d+)/;
-    $rows = 1 if $rows < 1;
-    $cols = 1 if $cols < 1;
-    $args->{matrix} = 1 unless defined $args->{matrix};
-    return [ map [ map {''} 1 .. $cols ], 1 .. $rows ];
+    $args->{_max_rows} = scalar @{ $data };
+    $args->{_max_cols} = scalar @{ $data->[0] };
+    $args->{_max_rows} = $fill{row} if ($fill{row} || 0) > ($args->{max_rows} || 0);
+    $args->{_max_cols} = $fill{col} if ($fill{col} || 0) > ($args->{max_cols} || 0);
+
+    return ( $self, Clone::clone($data), $args );
 }
 
 sub _expand_code_or_hash {
@@ -394,6 +394,8 @@ See L<Spreadsheet::HTML::Presets> for full documentation.
 
 =item * C<layout( %args )>
 
+=item * C<conway( %args )>
+
 =item * C<checkerboard( %args )>
 
 =item * C<chess( %args )>
@@ -402,6 +404,8 @@ See L<Spreadsheet::HTML::Presets> for full documentation.
 
 =item * C<dk( %args )>
 
+=item * C<shroom( %args )>
+
 =back
 
 =head1 PARAMETERS
