@@ -3,9 +3,10 @@ use strict;
 use warnings FATAL => 'all';
 
 use Spreadsheet::HTML;
+use Spreadsheet::HTML::Presets::Conway;
 
 sub layout {
-    Spreadsheet::HTML::generate( @_,
+    Spreadsheet::HTML::generate(
         encodes => '',
         matrix  => 1,
         table   => {
@@ -13,25 +14,19 @@ sub layout {
             ( map {$_ => 0} qw( border cellspacing cellpadding ) ),
         },
         _layout => 1,
+        @_,
     );
 }
 
 sub checkerboard {
     my $colors = _extract_arg( colors => @_ );
 
-    Spreadsheet::HTML::generate( @_,
-        td => { style  => { 'background-color' => $colors } }
+    Spreadsheet::HTML::generate(
+        td => { style  => { 'background-color' => $colors } },
+        @_,
     );
 }
 
-
-sub _extract_arg {
-    my ($arg) = shift;
-    for (0 .. $#_) {
-        return $_[$_+1] if $_[$_] eq $arg;
-    }
-}
-
 sub checkers {
     my @rows;
     $rows[0] = [ '', '&#9922;', '', '&#9922;', '', '&#9922;', '', '&#9922;', '', '&#9922;' ];
@@ -41,7 +36,7 @@ sub checkers {
     $rows[6] = [ '', '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;' ];
     $rows[7] = [ '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;', '' ];
 
-    Spreadsheet::HTML::generate( @_,
+    Spreadsheet::HTML::generate(
         tgroups  => 0,
         headless => 0,
         pinhead  => 0,
@@ -67,14 +62,15 @@ sub checkers {
                 border => 'thin inset',
                 'background-color' => [ ('white', 'red')x4, ('red', 'white')x4 ]
             }
-        }
+        },
+        @_,
     );
 }
 
 sub chess {
     my @black = ( '&#9820;', '&#9822;', '&#9821;', '&#9819;', '&#9818;', '&#9821;', '&#9822;', '&#9820;' );
     my @white = ( '&#9814;', '&#9816;', '&#9815;', '&#9813;', '&#9812;', '&#9815;', '&#9816;', '&#9814;' );
-    Spreadsheet::HTML::generate( @_,
+    Spreadsheet::HTML::generate(
         tgroups  => 0,
         headless => 0,
         pinhead  => 0,
@@ -98,7 +94,8 @@ sub chess {
                 border => 'thin inset',
                 'background-color' => [ ('white', '#aaaaaa')x4, ('#aaaaaa', 'white')x4 ]
             }
-        }
+        },
+        @_,
     );
 }
 
@@ -169,15 +166,113 @@ my $tmpl = '
         }
     }
 
-    layout( @_,
+    Spreadsheet::HTML::generate(
+        matrix   => 1,
         pinhead  => 0,
         tgroups  => 0,
         headless => 0,
         fill     => join( 'x', $total_rows, $total_cols ),
         @args,
+        @_,
     );
 }
 
+sub shroom {
+
+my $tmpl = '
+.....111111.....
+...1122223311...
+..133222233331..
+.13222222233331.
+.13223333222331.
+1222333333222221
+1222333333223321
+1322333333233331
+1332233332233331
+1332222222223321
+1322111111112221
+.11133133133111.
+..133313313331..
+..133333333331..
+...1333333331...
+....11111111....
+';
+
+    my %map = (
+        '.' => 'white',
+        1 => 'black',
+        2 => 'green',
+        3 => 'white',
+    );
+
+    my (@args);
+    my @lines = grep ! $_ =~ /^\s*$/, split /\n/, $tmpl;
+    my $total_rows = scalar @lines;
+    my $total_cols;
+    for my $row (0 .. $#lines) {
+        my @chars = split //, $lines[$row];
+        $total_cols ||= scalar @chars;
+        for my $col (0 .. $#chars) {
+            next unless my $color = $map{ $chars[$col] };
+            push @args, ( 
+                "-row${row}col${col}" => {
+                    width  => 16,
+                    height => 8,
+                    style  => { 'background-color' => $color },
+                }
+            );
+        }
+    }
+
+    Spreadsheet::HTML::generate(
+        matrix   => 1,
+        pinhead  => 0,
+        tgroups  => 0,
+        headless => 0,
+        fill     => join( 'x', $total_rows, $total_cols ),
+        @args,
+        @_,
+    );
+}
+
+sub conway {
+    my ($self,$data,$args) = Spreadsheet::HTML::_args( @_ );
+    $args->{on}    ||= '#00BFA5';
+    $args->{off}   ||= '#EEE';
+
+    my @args;
+    for my $r ( 0 .. $args->{_max_rows} - 1 ) {
+        for my $c ( 0 .. $args->{_max_cols} - 1 ) {
+            push @args,
+                sprintf( "-row%scol%s", $r, $c ) => {
+                    id     => join( '-', $r, $c ),
+                    class  => 'conway',
+                    width  => '30px',
+                    height => '30px',
+                    style  => { 'background-color' => $args->{off} },
+                };
+        }
+    }
+
+    Spreadsheet::HTML::Presets::Conway::_javascript( $args->{_max_rows}, $args->{_max_cols}, $args->{off}, $args->{on} ) .
+    Spreadsheet::HTML::generate(
+        pinhead  => 0,
+        tgroups  => 0,
+        headless => 0,
+        matrix   => 1,
+        caption  => { '<button onClick="start()">start</button>' => { align => 'bottom' } },
+        @args,
+        @_,
+    );
+}
+
+sub _extract_arg {
+    my ($arg) = shift;
+    for (0 .. $#_) {
+        return $_[$_+1] if $_[$_] eq $arg;
+    }
+}
+
 =head1 NAME
 
 Spreadsheet::HTML::Presets - Preset tables for fun and games.
@@ -188,7 +283,7 @@ See L<Spreadsheet::HTML>
 
 =over 4
 
-=item * C<layout()>
+=item * C<layout( %args )>
 
 Layout tables are not recommended, but if you choose to
 use them you should label them as such. This adds W3C
@@ -196,19 +291,30 @@ recommended layout attributes to the table tag and features:
 emiting only <td> tags, no padding or pruning of rows, forces
 no HTML entity encoding in table cells.
 
-=item * C<checkerboard( colors => [ ] )>
+=item * C<checkerboard( colors, %args )>
 
 Preset for tables with checkerboard colors.
 
-=item * C<checkers()>
+  checkerboard( data => [], colors => [qw(red green orange)] )
+
+=item * C<conway( on, off, %args )>
+
+Game of life. Current Javascript implementation
+Copyright 2015 Sandeep kumar H R.
+
+  conway( fill => '8x8', on => 'red' )
+
+=item * C<checkers( %args )>
 
 Generates a static checkers game board (US).
 
-=item * C<chess()>
+=item * C<chess( %args )>
 
 Generates a static chess game board.
 
-=item * C<dk()>
+=item * C<dk( %args )>
+
+=item * C<shroom( %args )>
 
 =back
 
